from django.core.management.base import BaseCommand

from hshassets import utils

import os
import traceback
from watchdog import events
from watchdog.observers import Observer


class AssetBuildHandler(events.PatternMatchingEventHandler):

    def __init__(self, scss_include_paths, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.scss_include_paths = scss_include_paths

    def on_modified(self, event):
        if isinstance(event, events.FileModifiedEvent):
            print('File modified: \033[35m{}\033[0m'.format(event.src_path))
            try:
                app_name, app_directories = utils.discover_app(event.src_path)
                utils.build_specific(app_name, app_directories, event.src_path, self.scss_include_paths)
            except Exception as e:
                print('Failed! \033[31m(✖_✖)\033[0m\n')
                traceback.print_exc()


class ImageHandler(events.PatternMatchingEventHandler):

    def dispatch(self, event):
        print('Image change detected, \033[35m{}\033[0m'.format(event.src_path))
        app_name, app_directories = utils.discover_app(event.src_path)
        utils.copy_images(app_directories)


class FontHandler(events.PatternMatchingEventHandler):

    def dispatch(self, event):
        print('Font change detected, \033[35m{}\033[0m'.format(event.src_path))
        app_name, app_directories = utils.discover_app(event.src_path)
        utils.copy_fonts(app_directories)


class Command(BaseCommand):
    help = 'Watch assets steadily and build them if necessary'

    def handle(self, *args, **options):
        print('Building assets initially ...')
        utils.do_everything()
        print('Done. Watching now for changes \033[94m(ಠ_ಠ)\033[0m ...\n')

        observer = Observer()

        asset_directories = utils.get_asset_directories()
        scss_include_paths = utils.get_scss_include_paths(asset_directories)

        build_handler = AssetBuildHandler(scss_include_paths, patterns=['*.js', '*.sass', '*.scss', '*.css'])
        image_handler = ImageHandler(patterns=['*'])
        for app_name, app_directories in asset_directories.items():

            if not os.path.isdir(app_directories['static_path']):
                os.makedirs(app_directories['static_path'])

            observer.schedule(build_handler, app_directories['asset_path'], recursive=True)

            if os.path.isdir(app_directories['asset_path'] + '/img'):
                observer.schedule(image_handler, app_directories['asset_path'] + '/img', recursive=True)

        observer.start()

        try:
            observer.join()
        except KeyboardInterrupt:
            print('\033[2DBye \033[95mʕʘ‿ʘʔ\033[0m')
