from django.conf import settings

from hshassets import logger
from hshassets.app_settings import COLOR_SCHEME, SCSS_INCLUDE_PATHS, STATIC_TARGET_APPS

from importlib import import_module

import os
import time
import sass
import jsmin
import shutil


def get_asset_directories():
    """get every app path that contains an "asset" folder on its root"""

    asset_apps = {}
    all_apps = {}

    for app_name in settings.INSTALLED_APPS:
        app_module = import_module(app_name)
        app_path = app_module.__path__[0]

        if os.path.isdir(app_path + '/assets'):
            asset_apps.update({
                app_name: {
                    'app_path': app_path,
                    'static_base_path': app_path + '/static/',
                    'static_path': app_path + '/static/' + app_name,
                    'asset_path': app_path + '/assets'
                }
            })

        all_apps.update({
            app_name: {
                'app_path': app_path,
                'static_base_path': app_path + '/static/',
                'static_path': app_path + '/static/' + app_name,
                'asset_path': app_path + '/assets'
            }
        })

    for source_app, target_app in STATIC_TARGET_APPS.items():
        asset_apps[source_app]['static_path'] = all_apps[target_app]['static_base_path'] + source_app

    return asset_apps


def get_scss_include_paths(asset_directories):
    """ Get additional include_paths for the scss parser - to be able to load e.g. bulma without getting insane """
    include_paths = []
    for app_name, rel_paths in SCSS_INCLUDE_PATHS:
        if app_name in asset_directories.keys():
            app_meta = asset_directories[app_name]
            for rel_path in rel_paths:
                lib_path = os.path.join(app_meta['asset_path'], rel_path)
                if os.path.exists(lib_path):
                    include_paths.append(lib_path)
                else:
                    logger.warning('You specified an additional scss path which can not be found: {}'.format(lib_path))
        else:
            logger.warning(
                'You specified the app {} in "SCSS_INCLUDE_PATHS" that can not be found. Is it in INSTALLED_APPS?'
                .format(app_name)
            )
    return include_paths


def insert_corporate_design_import(filepath):
    content = ''
    tmp_filename = '/sass/_cd_init.tmp.scss'

    with open(filepath + '/sass/_init.scss', 'r') as scss_file:
        content = scss_file.read()

    content = '@import "cd/{}.scss";\n\n'.format(COLOR_SCHEME) + content

    with open(filepath + tmp_filename, 'w') as tmp_scss_file:
        tmp_scss_file.write(content)

    return filepath + tmp_filename


def remove_corporate_design_import(filepath):
    os.remove(filepath)


def build_scss(app_name, app_directories, include_paths, verbose=True):
    if verbose:
        print('Building \033[36msass\033[0m for app "{}" ...'.format(app_name))

    start = time.time()

    init_file_path = app_directories['asset_path'] + '/sass/_init.scss'

    if not os.path.isdir(os.path.dirname(init_file_path)):
        return
    elif not os.path.isfile(init_file_path):
        print('Init scss file not found! Searched for "{}"'.format(init_file_path))
        return

    with open(init_file_path, 'r') as init_scss_file:
        content = init_scss_file.read()

    if app_name == 'hshassets':
        style_file = app_directories['asset_path'] + '/sass/cd/{}.scss'.format(COLOR_SCHEME)

        if os.path.isfile(style_file):
            with open(style_file, 'r') as cdfile:
                content = cdfile.read() + '\n\n' + content

    # add this apps sass folder to the include paths
    include_paths = [os.path.join(app_directories['asset_path'], 'sass')] + include_paths

    expanded_output = sass.compile(string=content, include_paths=include_paths, output_style='expanded')
    compressed_output = sass.compile(string=content, include_paths=include_paths, output_style='compressed')

    with open(app_directories['static_path'] + '/styles.css', 'w') as outfile:
        outfile.write(expanded_output)
    with open(app_directories['static_path'] + '/styles.min.css', 'w') as outfile:
        outfile.write(compressed_output)

    if verbose:
        print('Finished after {:.3f} seconds\n'.format(time.time() - start))


def build_javascript(app_name, app_directories, verbose=True):
    if verbose:
        print('Building \033[33mjavascript\033[0m for app "{}" ...'.format(app_name))

    start = time.time()

    js_files = []
    for root, dirs, files in os.walk(app_directories['asset_path'] + '/js'):
        for f in files:
            if f.endswith('.js'):
                js_files.append(root + '/' + f)

    for js_file in js_files:
        with open(js_file, 'r') as infile:
            content = infile.read()

            dest_js_path = app_directories['static_path'] + '/js/'
            dest_file_name = os.path.basename(js_file)
            dest_file_name_stripped, dest_file_extenstion = os.path.splitext(dest_file_name)

            if not os.path.isdir(dest_js_path):
                os.makedirs(dest_js_path)

            with open(dest_js_path + dest_file_name, 'w') as outfile:
                outfile.write(content + '\n')

            with open(dest_js_path + dest_file_name_stripped + '.min' + dest_file_extenstion, 'w') as outfile:
                outfile.write(jsmin.jsmin(content))

    if verbose:
        print('Finished after {:.3f} seconds\n'.format(time.time() - start))


def copy_images(app_name, app_directories, verbose=True):
    if not os.path.isdir(app_directories['asset_path'] + '/img'):
        return

    if app_name == 'hshassets':
        shutil.copy2(
            app_directories['asset_path'] + '/img/hsh_brand/favicons/{}/favicon.ico'.format(COLOR_SCHEME),
            app_directories['static_path'] + '/../favicon.ico'  # favicon for every app
        )

    start = time.time()
    if verbose:
        print('Copying images ...')

    if os.path.isdir(app_directories['static_path'] + '/img'):
        shutil.rmtree(app_directories['static_path'] + '/img')

    shutil.copytree(
        app_directories['asset_path'] + '/img',
        app_directories['static_path'] + '/img'
    )

    if verbose:
        print('Finished after {:.3f} seconds\n'.format(time.time() - start))


def copy_fonts(app_name, app_directories, verbose=True):
    if not os.path.isdir(app_directories['asset_path'] + '/fonts'):
        return

    start = time.time()
    if verbose:
        print('Copying fonts ...')

    if os.path.isdir(app_directories['static_path'] + '/fonts'):
        shutil.rmtree(app_directories['static_path'] + '/fonts')

    shutil.copytree(
        app_directories['asset_path'] + '/fonts',
        app_directories['static_path'] + '/fonts'
    )

    if verbose:
        print('Finished after {:.3f} seconds\n'.format(time.time() - start))


def discover_app(file_path):
    app_directories = {}

    for app_name, current_app_directories in get_asset_directories().items():
        if file_path.startswith(current_app_directories['app_path']):
            app_directories = current_app_directories
            break

    return app_name, app_directories


def do_everything(verbose=False):
    asset_directories = get_asset_directories()
    include_paths = get_scss_include_paths(asset_directories)
    for app_name, app_directories in asset_directories.items():

        if os.path.isdir(app_directories['static_path']):
            shutil.rmtree(app_directories['static_path'])

        os.makedirs(app_directories['static_path'])

        build_scss(app_name, app_directories, include_paths, verbose=verbose)
        build_javascript(app_name, app_directories, verbose=verbose)
        copy_images(app_name, app_directories, verbose=verbose)
        copy_fonts(app_name, app_directories, verbose=verbose)


def build_specific(app_name, app_directories, file_path, scss_include_paths):
    if file_path.endswith(('.sass', '.scss', '.css')):
        build_scss(app_name, app_directories, scss_include_paths)
    elif file_path.endswith(('.js')):
        build_javascript(app_name, app_directories)
